#include <avr/io.h>
#include "Defines.h"
#include "Key_Enum.h"
#include "InputProc_Function_Prototypes.h"

bool GetInputE(KeyPressed* pKeyPressed, unsigned char debouncedStateE)
{
	// i represents the pin to be checked.
	for (int i = 0; i < 8; i++)
	{
		if (!(debouncedStateE & (1 << i)) && *pKeyPressed == None)
		{
			switch (i)
			{
				case 0:
					*pKeyPressed = Num0;
					return true;

				case 1:
					*pKeyPressed = Num1;
					return true;

				case 2:
					*pKeyPressed = Num2;
					return true;

				case 3:
					*pKeyPressed = Num3;
					return true;

 				case 4:
					*pKeyPressed = Num4;
					return true;

 				case 5:
					*pKeyPressed = Num5;
					return true;

 				case 6:
					*pKeyPressed = Num6;
					return true;

 				case 7:
					*pKeyPressed = Num7;
					return true;
			}
		}
		else if (debouncedStateE & (1 << i) && *pKeyPressed - 1 == i)
		{
			// If we arrive here, it means that the bit being checked is NOT set,
			// i.e. the button is not pressed or debounced BUT *pKeyPressed says that it is.
			// This means the key was previously pressed and was released.
			*pKeyPressed = None;
		}
	}

	// No changes in keys being pressed (on this port). Some might have been released.
	return false;
}

bool GetInputF(KeyPressed* pKeyPressed, unsigned char debouncedStateF)
{
	// i represents the pin to be checked.
	for (int i = 0; i < 8; i++)
	{
		if (!(debouncedStateF & (1 << i)) && *pKeyPressed == None)
		{
			switch (i)
			{
				case 0:
					*pKeyPressed = Num8;
					return true;

				case 1:
					*pKeyPressed = Num9;
					return true;

				case 2:
					*pKeyPressed = Plus;
					return true;

				case 3:
					*pKeyPressed = Minus;
					return true;

 				case 4:
					*pKeyPressed = Multiply;
					return true;

 				case 5:
					*pKeyPressed = Divide;
					return true;

 				case 6:
					*pKeyPressed = LeftParenthesis;
					return true;

 				case 7:
					*pKeyPressed = RightParenthesis;
					return true;
			}
		}
		else if (debouncedStateF & (1 << i) && *pKeyPressed - Num8 == i)
		{
			// Notice + Num8 here. Num8 and up are the integer values 9 and up in the enum KeyPressed.
			// If we arrive here, it means that the bit being checked is NOT set,
			// i.e. the button is not pressed or debounced BUT *pKeyPressed says that it is.
			// This means the key was previously pressed and was released.
			*pKeyPressed = None;
		}
	}

	// No changes in keys being pressed (on this port). Some might have been released.
	return false;
}	

// This function processes input.
void InputProc(KeyPressed keyPressed, unsigned char* pInputString, unsigned char* pStringSz)
{
	// "None" case cannont be handles as the function is never called under such a circumstance.
	switch (keyPressed)
	{
		case Num0:
			PORTB = 0x3F;
			pInputString[*pStringSz] = '0';
			break;
		
		case Num1:
			PORTB = 0x06;
			pInputString[*pStringSz] = '1';
			break;

		case Num2:
			PORTB = 0x5B;
			pInputString[*pStringSz] = '2';
			break;
		
		case Num3:
			PORTB = 0x4F;
			pInputString[*pStringSz] = '3';
			break;

		case Num4:
			PORTB = 0x66;
			pInputString[*pStringSz] = '4';
			break;
		
		case Num5:
			PORTB = 0x6D;
			pInputString[*pStringSz] = '5';
			break;

		case Num6:
			PORTB = 0x7D;
			pInputString[*pStringSz] = '6';
			break;
		
		case Num7:
			PORTB = 0x07;
			pInputString[*pStringSz] = '7';
			break;

		case Num8:
			PORTB = 0x7F;
			pInputString[*pStringSz] = '8';
			break;
		
		case Num9:
			PORTB = 0x6F;
			pInputString[*pStringSz] = '9';
			break;

		case Plus:
			PORTB = 0x73;
			pInputString[*pStringSz] = '+';
			break;

		case Minus:
			PORTB = 0x40;
			pInputString[*pStringSz] = '-';
			break;

		case Multiply:
			PORTB = 0x80;
			pInputString[*pStringSz] = '*';
			break;

		case Divide:
			PORTB = 0x52;
			pInputString[*pStringSz] = '/';
			break;

		case LeftParenthesis:
			PORTB = 0x39;
			pInputString[*pStringSz] = '(';
			break;

		case RightParenthesis:
			PORTB = 0x0F;
			pInputString[*pStringSz] = ')';
			break;
	}

	// Increment size by 1.
	(*pStringSz)++;
}
