#include <avr/io.h>
#include <avr/interrupt.h>
#include "Defines.h"
#include "Timer0_Function_Prototypes.h"

#define MAXCHECKS 10

// state is used to compare the last MAXCHECKS states of the input port.
// index is used to index the array in the ISR.
volatile unsigned char statePORTE[MAXCHECKS];
volatile unsigned char statePORTF[MAXCHECKS];
volatile unsigned char stateIndex;

void InitTimer(void)
{
	// Fill arrays with 1's.
	for (int i = 0; i < MAXCHECKS; i++)
		statePORTE[i] = 0xFF;

	for (int i = 0; i < MAXCHECKS; i++)
		statePORTF[i] = 0xFF;

	stateIndex = 0;

	// Set register bits.
	TCCR0A = 0x02;
	TCCR0B = 0x04;
	OCR0A = 62;
	TIMSK0 = 0x02;
}

// This function is used when you want to know which switches are debounced or not (PORTE).
char GetDebouncedStateE(void)
{
	char debouncedState = 0x00;

	for (int i = 0; i < MAXCHECKS; i++)
		debouncedState |= statePORTE[i];

	return debouncedState;
}

// This function is used when you want to know which switches are debounced or not (PORTF).
char GetDebouncedStateF(void)
{
	char debouncedState = 0x00;

	for (int i = 0; i < MAXCHECKS; i++)
		debouncedState |= statePORTF[i];

	return debouncedState;
}

// Interrupt Service Routine.
ISR(TIMER0_COMPA_vect)
{
	// The ISR only updates the state array. Use GetDebouncedState to get current debounced status on port.
	// Very important: Only increment stateIndex after the LAST array has had it's value set!
	statePORTE[stateIndex] = PINE;
	statePORTF[stateIndex++] = PINF;

	if (stateIndex >= MAXCHECKS)
		stateIndex = 0;
}
