#include <avr/io.h>
#include <avr/interrupt.h>
#include "Defines.h"
#include "Timer0_Function_Prototypes.h"
#include "Port_Initialization_Function_Prototypes.h"
#include "InputProc_Function_Prototypes.h"
#include "Output_Function_Prototype.h"
#include "Key_Enum.h"

#define MAX_STRING_SIZE 20

int main(void)
{
	InitPorts();
	InitTimer();
	sei();

	// No key is pressed at the beginning.
	KeyPressed keyPressed = None;
	unsigned char debouncedStateE = 0xFF;
	unsigned char debouncedStateF = 0xFF;
	
	// Mode variable.
	unsigned char mode = 0;

	// String to store characters.
	unsigned char inputString[MAX_STRING_SIZE];
	unsigned char stringSz = 0;

	PORTL = 0x01;

	while (true)
	{
		while (mode == 0)
		{
			debouncedStateE = GetDebouncedStateE();
			debouncedStateF = GetDebouncedStateF();
		
			if (GetInputE(&keyPressed, debouncedStateE))
				InputProc(keyPressed, inputString, &stringSz);	
			else if (GetInputF(&keyPressed, debouncedStateF))
				InputProc(keyPressed, inputString, &stringSz);
			
			// If the max string size has been reached, switch to output mode.
			if (stringSz > MAX_STRING_SIZE - 2)
			{
				mode = 1;
			}					
		}

		// Append terminating null character to string.
		inputString[MAX_STRING_SIZE - 1] = '\0';

		PORTL = 0x80;

		while (mode == 1)
		{
			OutputString(inputString, stringSz);
		}
	}

	return 0;
}
